"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const test_1 = require("@schematics/angular/utility/test");
const index_1 = require("../index");
function addGoogleAnalytics(options) {
    const source = schematics_1.apply(schematics_1.url('../utils/googleAnalytics/files'), [
        schematics_1.move(`${options.directory}/src/bootstrapScripts`)
    ]);
    return schematics_1.chain([
        schematics_1.mergeWith(source),
        tree => {
            let indexContent = '';
            if (tree.exists(`${options.directory}/src/bootstrapScripts/index.ts`)) {
                indexContent = test_1.getFileContent(tree, `${options.directory}/src/bootstrapScripts/index.ts`);
            }
            index_1.createOrOverwriteFile(tree, `${options.directory}/src/bootstrapScripts/index.ts`, indexContent + 'export * from \'./googleAnalytics\';\n');
            //find and overwrite environment files
            tree.getDir(`${options.directory}/src/environments`).visit((path) => {
                if (path.endsWith('.ts')) {
                    index_1.addEntryToEnvironment(tree, path, 'gaTrackingCode', options.gaTrackingCode);
                    index_1.addEntryToEnvironment(tree, path, 'googleAnalytics', path.indexOf('prod') > -1);
                }
            });
            //find and add code to the main browser file
            const cliConfig = JSON.parse(test_1.getFileContent(tree, `${options.directory}/angular.json`));
            const mainFilePath = `${options.directory}/` + cliConfig.projects[options.name].architect.build.options.main;
            index_1.addImportStatement(tree, mainFilePath, 'import { googleAnalytics } from \'./bootstrapScripts\';');
            // adding script at the end of file
            const sourceText = test_1.getFileContent(tree, mainFilePath);
            const changeRecorder = tree.beginUpdate(mainFilePath);
            changeRecorder.insertRight(sourceText.length, 'googleAnalytics();\n');
            tree.commitUpdate(changeRecorder);
            return tree;
        }
    ]);
}
exports.addGoogleAnalytics = addGoogleAnalytics;
//# sourceMappingURL=index.js.map